import unittest
from dataclasses import dataclass

import pandas

from autofaker import autodata, autopandas, fakedata, fakepandas


@dataclass
class DataClass:
    id: int
    name: str
    text: str


@autodata(str, str, str)
def autodata_test_method(a, b, c):
    pass


@fakedata(str, str, str)
def fakedata_test_method(a, b, c):
    pass


@autopandas(DataClass)
def autopandas_test_method(df: pandas.DataFrame):
    pass


class AnonymousPrimitivesViaDecoratorThrowsExceptionsTests(unittest.TestCase):
    def test_autodata_throws_error_when_decorating_non_testmethod(self):
        with self.assertRaises(NotImplementedError):

            class X:
                @autodata(str, str, str)
                def not_test_method(self):
                    pass

            X().not_test_method()

    def test_autodata_throws_error_when_used_without_arguments(self):
        with self.assertRaises(NotImplementedError):

            class X:
                @autodata()
                def not_test_method(self):
                    pass

            X().not_test_method()

    def test_autodata_throws_error_when_used_without_arguments_annotation(self):
        with self.assertRaises(ValueError):

            class X(unittest.TestCase):
                @autodata()
                def not_test_method(self, text, number, decimal):
                    pass

            X().not_test_method()

    def test_fakedata_throws_error_when_decorating_non_testmethod(self):
        with self.assertRaises(NotImplementedError):

            class X:
                @fakedata(str, str, str)
                def not_test_method(self):
                    pass

            X().not_test_method()

    def test_fakedata_throws_error_when_used_without_arguments(self):
        with self.assertRaises(NotImplementedError):

            class X:
                @fakedata()
                def not_test_method(self):
                    pass

            X().not_test_method()

    def test_fakedata_throws_error_when_used_without_arguments_annotation(self):
        with self.assertRaises(ValueError):

            class X(unittest.TestCase):
                @fakedata()
                def not_test_method(self, text, number, decimal):
                    pass

            X().not_test_method()

    def test_autopandas_throws_error_when_decorating_non_testmethod(self):
        class SimpleClass:
            id = -1
            name = "name"
            text = "test"

        with self.assertRaises(NotImplementedError):

            class X:
                @autopandas(SimpleClass)
                def not_test_method(self, df):
                    pass

            X().not_test_method()

    def test_fakepandas_throws_error_when_decorating_non_testmethod(self):
        class SimpleClass:
            id = -1
            name = "name"
            text = "test"

        with self.assertRaises(NotImplementedError):

            class X:
                @fakepandas(SimpleClass)
                def not_test_method(self, df):
                    pass

            X().not_test_method()

    def test_autodata_doesnt_throw_error_when_decorating_non_pytest(self):
        autodata_test_method()

    def test_fakedata_doesnt_throw_error_when_decorating_non_pytest(self):
        fakedata_test_method()

    def test_autopandas_doesnt_throw_error_when_decorating_non_pytest(self):
        autopandas_test_method()



from datetime import date, datetime

from autofaker import autodata, fakedata

# Anonymous Primitives Via Decorator Tests


@autodata(str)
def test_create_str_using_decorator(text):
    assert text is not None


@autodata(int)
def test_create_int_using_decorator(number):
    assert None is not number


@autodata(float)
def test_create_float_using_decorator(number):
    assert None is not number


@autodata(bool)
def test_create_boolean_using_decorator(boolean):
    assert None is not boolean


@autodata(complex)
def test_create_complex_using_decorator(complex_type):
    assert None is not complex_type


@autodata(range)
def test_create_range_using_decorator(r):
    assert None is not r


@autodata(bytes)
def test_create_bytes_using_decorator(b):
    assert None is not b


@autodata(bytearray)
def test_create_bytearray_using_decorator(b):
    assert None is not b


@autodata(memoryview)
def test_create_memoryview_using_decorator(b):
    assert None is not b


@autodata(datetime)
def test_create_datetime_using_decorator(dt):
    assert None is not dt


@autodata(date)
def test_create_date_using_decorator(d):
    assert None is not d


# Anonymous Primitives Via Decorator With Fakes


@autodata(str, use_fake_data=True)
def test_create_str_using_decorator_with_fake_data(text):
    assert text is not None


@autodata(int, use_fake_data=True)
def test_create_int_using_decorator_with_fake_data(number):
    assert None is not number


@autodata(float, use_fake_data=True)
def test_create_float_using_decorator_with_fake_data(number):
    assert None is not number


@autodata(bool, use_fake_data=True)
def test_create_boolean_using_decorator_with_fake_data(boolean):
    assert None is not boolean


@autodata(complex, use_fake_data=True)
def test_create_complex_using_decorator_with_fake_data(complex_type):
    assert None is not complex_type


@autodata(range, use_fake_data=True)
def test_create_range_using_decorator_with_fake_data(r):
    assert None is not r


@autodata(bytes, use_fake_data=True)
def test_create_bytes_using_decorator_with_fake_data(b):
    assert None is not b


@autodata(bytearray, use_fake_data=True)
def test_create_bytearray_using_decorator_with_fake_data(b):
    assert None is not b


@autodata(memoryview, use_fake_data=True)
def test_create_memoryview_using_decorator_with_fake_data(b):
    assert None is not b


@autodata(datetime, use_fake_data=True)
def test_create_datetime_using_decorator_with_fake_data(dt):
    assert None is not dt


@autodata(date, use_fake_data=True)
def test_create_date_using_decorator_with_fake_data(d):
    assert None is not d


# Multiple Anonymous Primitives Via Decorator Tests


@autodata(str, int, float, complex, range, bytes, bytearray, memoryview)
def test_create_primitives_using_decorator(
    text, number, decimal, complex_type, range_type, buffer, buffer2, memview
):
    assert text is not None
    assert number != 0
    assert decimal != float()
    assert complex_type != complex()
    assert range_type != range(0)
    assert buffer != bytes()
    assert buffer2 != bytearray()
    assert memview != memoryview(bytes())


@autodata(
    str, int, float, complex, range, bytes, bytearray, memoryview, use_fake_data=True
)
def test_create_primitives_using_decorator_with_fakes(
    text, number, decimal, complex_type, range_type, buffer, buffer2, memview
):
    assert text is not None
    assert number != 0
    assert decimal != float()
    assert complex_type != complex()
    assert range_type != range(0)
    assert buffer != bytes()
    assert buffer2 != bytearray()
    assert memview != memoryview(bytes())


# Multiple Arguments Via Decorator Tests


@fakedata()
def test_create_anonymous_arguments_using_decorator(
    text: str,
    number: int,
    decimal: float,
    complex_type: complex,
    range_type: range,
    buffer: bytes,
    buffer2: bytearray,
    memview: memoryview,
):
    print(text)
    print(number)
    print(decimal)
    print(complex_type)
    assert text is not None
    assert number != 0
    assert decimal != float()
    assert complex_type != complex()
    assert range_type != range(0)
    assert buffer != bytes()
    assert buffer2 != bytearray()
    assert memview != memoryview(bytes())


@fakedata()
def test_create_fake_arguments_using_decorator(
    text: str,
    number: int,
    decimal: float,
    complex_type: complex,
    range_type: range,
    buffer: bytes,
    buffer2: bytearray,
    memview: memoryview,
):
    print(text)
    print(number)
    print(decimal)
    print(complex_type)
    assert text is not None
    assert number != 0
    assert decimal != float()
    assert complex_type != complex()
    assert range_type != range(0)
    assert buffer != bytes()
    assert buffer2 != bytearray()
    assert memview != memoryview(bytes())


@autodata()
def test_create_arguments_using_decorator(
    text: str,
    number: int,
    decimal: float,
    complex_type: complex,
    range_type: range,
    buffer: bytes,
    buffer2: bytearray,
    memview: memoryview,
):
    print(text)
    print(number)
    print(decimal)
    print(complex_type)
    assert text is not None
    assert number != 0
    assert decimal != float()
    assert complex_type != complex()
    assert range_type != range(0)
    assert buffer != bytes()
    assert buffer2 != bytearray()
    assert memview != memoryview(bytes())


@autodata(use_fake_data=True)
def test_create_arguments_using_decorator_with_fakes(
    text: str,
    number: int,
    decimal: float,
    complex_type: complex,
    range_type: range,
    buffer: bytes,
    buffer2: bytearray,
    memview: memoryview,
):
    print(text)
    print(number)
    print(decimal)
    print(complex_type)
    assert text is not None
    assert number != 0
    assert decimal != float()
    assert complex_type != complex()
    assert range_type != range(0)
    assert buffer != bytes()
    assert buffer2 != bytearray()
    assert memview != memoryview(bytes())



import dataclasses

import pandas as pd

from autofaker.attributes import Attributes
from autofaker.generator import TypeDataGenerator


class PandasDataFrameGenerator:
    def __init__(self, t, rows: int = 3, use_fake_data: bool = False):
        self.data = []
        for _ in range(rows):
            row = TypeDataGenerator.create(t, use_fake_data=use_fake_data).generate()
            self.data.append(row)

    def generate(self):
        members = [
            attr
            for attr in dir(self.data[0])
            if dataclasses.is_dataclass(getattr(self.data[0], attr))
            or not callable(getattr(self.data[0], attr))
            and not attr.startswith("__")
        ]
        rows = []
        for d in self.data:
            row = []
            for member in members:
                row.append(Attributes(d).get_attribute(member))
            rows.append(row)
        return pd.DataFrame(rows, columns=members)



import dataclasses
from dataclasses import dataclass

from autofaker import autodata, fakedata


@dataclass
class SimpleClass:
    id: int
    name: str
    text: str


@dataclass
class NestedClass:
    id: int
    name: str
    text: str
    inner: SimpleClass


@dataclass
class DoubleNestedClass:
    id: int
    name: str
    text: str
    inner: NestedClass


# Anonymous Nested Class Via Decorator Test Cases


@autodata(NestedClass)
def test_create_nested_class_using_decorator_returns_not_none(instance):
    assert instance is not None


@autodata(NestedClass)
def test_create_nested_class_using_decorator_returns_dataclass(instance):
    assert dataclasses.is_dataclass(instance)


@autodata(NestedClass)
def test_create_nested_class_returns_instance_with_new_values(instance):
    assert instance.id != 0
    assert instance.inner.id != 0
    assert instance.inner.text != str()


@autodata()
def test_create_nested_class_argument_returns_instance_with_new_values(
    instance: NestedClass,
):
    assert instance.id != 0
    assert instance.name != str()
    assert instance.text != str()


@autodata()
def test_create_anonymous_nested_class_returns_instance_with_new_values(
    instance: NestedClass,
):
    assert instance.id != 0
    assert instance.name != str()
    assert instance.text != str()


@fakedata()
def test_create_fake_nested_class_returns_instance_with_new_values(
    instance: NestedClass,
):
    assert instance.id != 0
    assert instance.name != str()
    assert instance.text != str()


# Anonymous Double Nested Class Via Decorator With Fakes Test Cases


@autodata(DoubleNestedClass, use_fake_data=True)
def test_create_double_nested_class_using_decorator_returns_not_none(instance):
    assert instance is not None


@autodata(DoubleNestedClass, use_fake_data=True)
def test_create_nested_class_using_decorator_returns_dataclass(instance):
    assert dataclasses.is_dataclass(instance)


@autodata(DoubleNestedClass, use_fake_data=True)
def test_create_double_nested_class_returns_instance_with_new_values(instance):
    assert instance.id != 0
    assert instance.inner.inner.id != 0
    assert instance.inner.inner.text != str()


@autodata(use_fake_data=True)
def test_create_double_nested_class_argument_using_decorator_returns_not_none(
    instance: DoubleNestedClass,
):
    assert instance is not None


@fakedata()
def test_create_double_nested_class_argument_returns_instance_with_new_values(
    instance: DoubleNestedClass,
):
    assert instance.id != 0
    assert instance.inner.inner.id != 0
    assert instance.inner.inner.text != str()



import unittest
from dataclasses import dataclass
from typing import List

from autofaker import Autodata, autodata, fakedata


class Calculator:
    def add(self, number1: int, number2: int):
        return number1 + number2


class CalculatorTests(unittest.TestCase):
    def test_can_add_two_numbers(self):
        numbers = Autodata.create_many(int, 2)
        sut = Autodata.create(Calculator)
        result = sut.add(numbers[0], numbers[1])
        self.assertEqual(numbers[0] + numbers[1], result)

    @autodata(Calculator, int, int)
    def test_can_add_two_numbers_using_test_arguments(self, sut, number1, number2):
        result = sut.add(number1, number2)
        self.assertEqual(number1 + number2, result)

    @autodata
    def test_can_add_two_numbers_using_annotated_arguments(
        self, sut: Calculator, number1: int, number2: int
    ):
        result = sut.add(number1, number2)
        self.assertEqual(number1 + number2, result)

    @autodata
    def test_can_add_two_numbers_using_anonymous_arguments(
        self, sut: Calculator, number1: int, number2: int
    ):
        result = sut.add(number1, number2)
        self.assertEqual(number1 + number2, result)

    @fakedata
    def test_can_add_two_numbers_using_fake_arguments(
        self, sut: Calculator, number1: int, number2: int
    ):
        result = sut.add(number1, number2)
        self.assertEqual(number1 + number2, result)


class Person:
    age = 10
    name = "test"

    def get_introduction(self):
        return f"Hi! My name is {self.name} and I'm {self.age} years old"


class Staff:
    def __init__(self):
        self.people = []

    def add_person(self, person: Person):
        self.people.append(person)

    def add_people(self, people: List[Person]):
        for person in people:
            self.add_person(person)

    def get_head_count(self):
        return len(self.people)

    def introduce(self):
        for person in self.people:
            print(person.get_introduction())


class StaffTests(unittest.TestCase):
    def test_introduce_everyone(self):
        people = Autodata.create_many(Person)
        sut = Autodata.create(Staff)
        sut.add_people(people)
        sut.introduce()
        self.assertEqual(len(people), sut.get_head_count())


class ConstructorWithPrimitiveArguments:
    def __init__(self, name: str, text: str, age: int):
        self.age = age
        self.text = text
        self.name = name


class ConstructorWithPrimitiveArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithPrimitiveArguments))

    def test_constructed_sut_age_not_none(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithPrimitiveArguments).age)

    def test_constructed_sut_name_not_none(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithPrimitiveArguments).name)

    def test_constructed_sut_text_not_none(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithPrimitiveArguments).text)


class ConstructorWithClassArguments:
    def __init__(self, cls: ConstructorWithPrimitiveArguments):
        self.inner = cls


class ConstructorWithClassArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithClassArguments))

    def test_constructed_sut_inner_not_none(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithClassArguments).inner)

    def test_constructed_sut_inner_has_correct_type(self):
        self.assertIsInstance(
            Autodata.create(ConstructorWithClassArguments).inner,
            ConstructorWithPrimitiveArguments,
        )


class ConstructorWithNestedClassArguments:
    def __init__(self, cls: ConstructorWithClassArguments):
        self.inner = cls


class ConstructorWithNestedClassArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithNestedClassArguments))

    def test_constructed_sut_inner_not_none(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithNestedClassArguments).inner)

    def test_constructed_sut_inner_has_correct_type(self):
        self.assertIsInstance(
            Autodata.create(ConstructorWithNestedClassArguments).inner,
            ConstructorWithClassArguments,
        )

    def test_constructed_sut_inner_inner_has_correct_type(self):
        self.assertIsInstance(
            Autodata.create(ConstructorWithNestedClassArguments).inner.inner,
            ConstructorWithPrimitiveArguments,
        )


class ConstructorWithDoubleNestedClassArguments:
    def __init__(self, cls: ConstructorWithNestedClassArguments):
        self.inner = cls


class ConstructorWithDoubleNestedClassArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithDoubleNestedClassArguments))

    def test_constructed_sut_inner_not_none(self):
        self.assertIsNotNone(
            Autodata.create(ConstructorWithDoubleNestedClassArguments).inner
        )

    def test_constructed_sut_inner_has_correct_type(self):
        self.assertIsInstance(
            Autodata.create(ConstructorWithDoubleNestedClassArguments).inner,
            ConstructorWithNestedClassArguments,
        )

    def test_constructed_sut_inner_inner_has_correct_type(self):
        self.assertIsInstance(
            Autodata.create(ConstructorWithDoubleNestedClassArguments).inner.inner,
            ConstructorWithClassArguments,
        )

    def test_constructed_sut_inner_inner_inner_has_correct_type(self):
        self.assertIsInstance(
            Autodata.create(
                ConstructorWithDoubleNestedClassArguments
            ).inner.inner.inner,
            ConstructorWithPrimitiveArguments,
        )


@dataclass
class DataClass:
    id: int
    name: str
    age: int
    email: str


class ConstructorWithDataClassArguments:
    def __init__(self, data: DataClass):
        self.data = data


class ConstructorWithDataClassArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithDataClassArguments))

    def test_constructed_sut_id_not_none(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithDataClassArguments).data.id)

    def test_constructed_sut_age_not_none(self):
        self.assertIsNotNone(
            Autodata.create(ConstructorWithDataClassArguments).data.age
        )

    def test_constructed_sut_name_not_none(self):
        self.assertIsNotNone(
            Autodata.create(ConstructorWithDataClassArguments).data.name
        )

    def test_constructed_sut_email_not_none(self):
        self.assertIsNotNone(
            Autodata.create(ConstructorWithDataClassArguments).data.email
        )


class ConstructorWithListArguments:
    def __init__(self, texts: List[str], numbers: List[int]):
        self.numbers = numbers
        self.texts = texts


class ConstructorWithListArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithListArguments))


class ConstructorWithClassListArguments:
    def __init__(self, items: List[ConstructorWithPrimitiveArguments]):
        self.items = items


class ConstructorWithClassListArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithClassListArguments))


class ConstructorWithDataClassListArguments:
    def __init__(self, items: List[DataClass]):
        self.items = items


class ConstructorWithDataClassListArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithDataClassListArguments))



from dataclasses import dataclass

import pandas

from autofaker import autodata, autopandas, fakepandas


class SimpleClassA:
    id = -1
    name: "test"
    text = "test"


class SimpleClassB:
    def __init__(self, id: int, name: str, text: str):
        self.text = text
        self.name = name
        self.id = id


class SimpleClassC:
    def __init__(self, a: SimpleClassA, b: SimpleClassB):
        self.b = b
        self.a = a


# Anonymous Pandas DataFrame Via Decorator


@autopandas(SimpleClassA)
def test_create_anonymous_pandas_dataframe_returns_not_none(df: pandas.DataFrame):
    assert df is not None


@autopandas(SimpleClassA, 10)
def test_create_anonymous_pandas_dataframe_with_rowcount_returns_not_empty(
    df: pandas.DataFrame,
):
    assert len(df.index) != 0


@fakepandas(SimpleClassA)
def test_create_fake_pandas_dataframe_returns_not_none(df: pandas.DataFrame):
    assert df is not None


@fakepandas(SimpleClassA, 10)
def test_create_fake_pandas_dataframe_with_rowcount_returns_not_empty(
    df: pandas.DataFrame,
):
    assert len(df.index) != 0


@autopandas(SimpleClassA)
def test_can_create_anonymous_pandas_dataframes(cls):
    print(cls)
    assert not cls.empty


@autopandas(SimpleClassB)
def test_can_create_anonymous_pandas_dataframes_from_class_with_constructor_arguments(
    cls,
):
    print(cls)
    assert not cls.empty


@autopandas(SimpleClassC)
def test_can_create_anonymous_pandas_dataframes_from_class_with_constructor_class_arguments(
    cls,
):
    print(cls)
    assert not cls.empty


@dataclass
class DataClass:
    id: int
    name: str
    text: str


# Anonymous Pandas DataFrame Via Decorator From DataClass


@autopandas(DataClass)
def test_create_anonymous_pandas_dataframe_returns_not_none(df: pandas.DataFrame):
    assert df is not None


@autopandas(DataClass, 10)
def test_create_anonymous_pandas_dataframe_with_rowcount_returns_not_empty(
    df: pandas.DataFrame,
):
    assert len(df.index) != 0


@fakepandas(DataClass, 10)
def test_create_fake_pandas_dataframe_with_rowcount_returns_not_empty(
    df: pandas.DataFrame,
):
    assert len(df.index) != 0


@fakepandas(DataClass)
def test_create_fake_pandas_dataframe_returns_not_none(df: pandas.DataFrame):
    assert df is not None


# Autodata Decorator Ignores Pandas


@autodata()
def test_autodata_decorator_ignores_pandas_dataframe(df: pandas.DataFrame):
    assert df is None


@autodata()
def test_autodata_decorator_ignores_only_pandas_dataframe(
    text: str, df: pandas.DataFrame
):
    assert None is not text
    assert df is None



"""
Provides anonymous object creation functions to help minimize the setup/arrange phase when writing unit tests
"""

import inspect
import unittest
from typing import List

from autofaker import Autodata, PandasDataFrameGenerator


def autodata(*types: object, use_fake_data: bool = False):
    """
    Creates anonymous variable of the requested types and pass them as arguments to a unit test function

    Example:

    import unittest

    from autofaker import autodata

    class SampleTest(unittest.TestCase):
        @autodata(str, int, float, bool)

        def test_create_str_argument_using_decorator(self, text, number, decimal, boolean):
            self.assertIsNotNone(text)


    :param use_fake_data: bool - Set this to True to use Faker to generate data, otherwise False to generate anonymous data
    :param type types: tuple - The types to generate data. This is optional and will use the arguments from the function being decorated if not specified
    """

    def decorator(function):
        def wrapper(*args):
            if __get_class_that_defined_method(function) is None:
                return function(
                    *tuple(
                        __create_function_args(
                            function, *tuple(types), use_fake_data=use_fake_data
                        )
                    )
                )
            return function(
                __get_test_class(*args),
                *tuple(
                    __create_function_args(
                        function, *tuple(types), use_fake_data=use_fake_data
                    )
                )
            )

        return wrapper

    return decorator


def fakedata(*types: object):
    """
    Creates fake values for the variables of the requested types and pass them as arguments to a unit test function

    Example:

    import unittest

    from autofaker import fakedata

    class SampleTest(unittest.TestCase):
        @fakedata()

        def test_create_fake_arguments(self, text: str, number: int, decimal: float, boolean: bool):
            self.assertIsNotNone(text)


    :param types: object - The types to generate data. This is optional and will use the arguments from the function being decorated if not specified
    """

    def decorator(function):
        def wrapper(*args):
            if __get_class_that_defined_method(function) is None:
                return function(
                    *tuple(
                        __create_function_args(
                            function, *tuple(types), use_fake_data=True
                        )
                    )
                )
            return function(
                __get_test_class(*args),
                *tuple(
                    __create_function_args(function, *tuple(types), use_fake_data=True)
                )
            )

        return wrapper

    return decorator


def autopandas(t: object, rows: int = 3, use_fake_data: bool = False):
    """
    Create a Pandas DataFrame containing anonymous data with the specified number of rows (default 3)

    :param type t: object - The class that represents the DataFrame. This can be a plain old class or a @dataclass
    :param type rows: int - The number of rows to generate for the DataFrame (default 3)
    :param use_fake_data: bool - Set this to True to use Faker to generate data, otherwise False to generate anonymous data
    """

    def decorator(function):
        def wrapper(*args):
            pdf = PandasDataFrameGenerator(
                t, rows, use_fake_data=use_fake_data
            ).generate()
            if __get_class_that_defined_method(function) is None:
                return function(pdf)
            return function(__get_test_class(*args), pdf)

        return wrapper

    return decorator


def fakepandas(t, rows: int = 3):
    """
    Create a Pandas DataFrame containing fake data with the specified number of rows (default 3)

    :param type t: object - The class that represents the DataFrame. This can be a plain old class or a @dataclass
    :param type rows: int - The number of rows to generate for the DataFrame (default 3)
    """
    return autopandas(t, rows, use_fake_data=True)


def __get_test_class(*args):
    test_class = args[0]
    if issubclass(test_class.__class__, unittest.TestCase) is False:
        raise NotImplementedError(
            "This way of creating anonymous objects are only supported from unit tests"
        )
    return test_class


def __get_class_that_defined_method(meth):
    if inspect.isfunction(meth):
        cls = getattr(
            inspect.getmodule(meth),
            meth.__qualname__.split(".<locals>", 1)[0].rsplit(".", 1)[0],
            None,
        )
        if isinstance(cls, type):
            return cls
    return getattr(meth, "__objclass__", None)  # handle special descriptor objects


def __create_function_args(function, *types, use_fake_data: bool = False) -> List:
    values = []
    argtpes = inspect.getfullargspec(function)
    args = argtpes.annotations.values() if types is None or len(types) == 0 else types
    for t in args:
        value = Autodata.create(t, use_fake_data)
        values.append(value)
    pos = 1
    if __get_class_that_defined_method(function) is None:
        pos = 0
    if len(argtpes.args) - pos != len(values):
        raise ValueError(
            "Missing argument annotations. Please declare the type of every argument"
        )
    return values



from autofaker import autodata, fakedata


class SimpleClass:
    id = -1
    name = "name"
    text = "test"


# Anonymous Simple Class Via Decorator


@autodata(SimpleClass)
def test_create_simple_class_using_decorator_returns_not_none(instance):
    assert instance is not None


@autodata(SimpleClass)
def test_create_simple_class_using_decorator_returns_instance(instance):
    assert isinstance(instance, SimpleClass)


@autodata(SimpleClass)
def test_create_simple_class_returns_instance_with_new_values(instance):
    assert instance.id != SimpleClass().id
    assert instance.name != SimpleClass().name
    assert instance.text != SimpleClass().text


@autodata()
def test_create_simple_class_argument_returns_instance_with_new_values(
    instance: SimpleClass,
):
    assert instance.id != SimpleClass().id
    assert instance.name != SimpleClass().name
    assert instance.text != SimpleClass().text


@autodata()
def test_create_anonymous_simple_class_returns_instance_with_new_values(
    instance: SimpleClass,
):
    assert instance.id != SimpleClass().id
    assert instance.name != SimpleClass().name
    assert instance.text != SimpleClass().text


@fakedata()
def test_create_fake_simple_class_returns_instance_with_new_values(
    instance: SimpleClass,
):
    assert instance.id != SimpleClass().id
    assert instance.name != SimpleClass().name
    assert instance.text != SimpleClass().text


# Anonymous Simple Class Via Decorator With Fakes


@autodata(SimpleClass, use_fake_data=True)
def test_create_simple_class_using_decorator_with_fake_data_returns_not_none(instance):
    assert instance is not None


@autodata(SimpleClass, use_fake_data=True)
def test_create_simple_class_using_decorator_with_fake_data_returns_instance(instance):
    assert isinstance(instance, SimpleClass)


@autodata(SimpleClass, use_fake_data=True)
def test_create_simple_class_returns_instance_with_new_values_using_fake_data(instance):
    assert instance.id != SimpleClass().id
    assert instance.text != SimpleClass().text


@autodata(use_fake_data=True)
def test_create_simple_class_argument_returns_instance_with_new_values_using_fake_data(
    instance: SimpleClass,
):
    assert instance.id != SimpleClass().id
    assert instance.name != SimpleClass().name
    assert instance.text != SimpleClass().text


@fakedata()
def test_create_fake_simple_class_returns_instance_with_new_values_using_fake_data(
    instance: SimpleClass,
):
    assert instance.id != SimpleClass().id
    assert instance.name != SimpleClass().name
    assert instance.text != SimpleClass().text



import unittest

from autofaker import autodata, fakedata


class SimpleClass:
    id = -1
    name = "name"
    text = "test"


class AnonymousSimpleClassViaDecoratorTestCase(unittest.TestCase):
    @autodata(SimpleClass)
    def test_create_simple_class_using_decorator_returns_not_none(self, instance):
        self.assertIsNotNone(instance)

    @autodata(SimpleClass)
    def test_create_simple_class_using_decorator_returns_instance(self, instance):
        self.assertIsInstance(instance, SimpleClass)

    @autodata(SimpleClass)
    def test_create_simple_class_returns_instance_with_new_values(self, instance):
        self.assertNotEqual(instance.id, SimpleClass().id)
        self.assertNotEqual(instance.name, SimpleClass().name)
        self.assertNotEqual(instance.text, SimpleClass().text)

    @autodata()
    def test_create_simple_class_argument_returns_instance_with_new_values(
        self, instance: SimpleClass
    ):
        self.assertNotEqual(instance.id, SimpleClass().id)
        self.assertNotEqual(instance.name, SimpleClass().name)
        self.assertNotEqual(instance.text, SimpleClass().text)

    @autodata()
    def test_create_anonymous_simple_class_returns_instance_with_new_values(
        self, instance: SimpleClass
    ):
        self.assertNotEqual(instance.id, SimpleClass().id)
        self.assertNotEqual(instance.name, SimpleClass().name)
        self.assertNotEqual(instance.text, SimpleClass().text)

    @fakedata()
    def test_create_fake_simple_class_returns_instance_with_new_values(
        self, instance: SimpleClass
    ):
        self.assertNotEqual(instance.id, SimpleClass().id)
        self.assertNotEqual(instance.name, SimpleClass().name)
        self.assertNotEqual(instance.text, SimpleClass().text)


class AnonymousSimpleClassViaDecoratorWithFakesTestCase(unittest.TestCase):
    @autodata(SimpleClass, use_fake_data=True)
    def test_create_simple_class_using_decorator_returns_not_none(self, instance):
        self.assertIsNotNone(instance)

    @autodata(SimpleClass, use_fake_data=True)
    def test_create_simple_class_using_decorator_returns_instance(self, instance):
        self.assertIsInstance(instance, SimpleClass)

    @autodata(SimpleClass, use_fake_data=True)
    def test_create_simple_class_returns_instance_with_new_values(self, instance):
        self.assertNotEqual(instance.id, SimpleClass().id)
        self.assertNotEqual(instance.text, SimpleClass().text)

    @autodata(use_fake_data=True)
    def test_create_simple_class_argument_returns_instance_with_new_values(
        self, instance: SimpleClass
    ):
        self.assertNotEqual(instance.id, SimpleClass().id)
        self.assertNotEqual(instance.name, SimpleClass().name)
        self.assertNotEqual(instance.text, SimpleClass().text)

    @fakedata()
    def test_create_fake_simple_class_returns_instance_with_new_values(
        self, instance: SimpleClass
    ):
        self.assertNotEqual(instance.id, SimpleClass().id)
        self.assertNotEqual(instance.name, SimpleClass().name)
        self.assertNotEqual(instance.text, SimpleClass().text)



from autofaker import autodata, fakedata


class SimpleClass:
    id = -1
    name = "name"
    text = "test"


class NestedClass:
    id = -1
    name = "name"
    text = "test"
    inner = SimpleClass()


class DoubleNestedClass:
    id = -1
    name = "name"
    text = "test"
    inner = NestedClass()


# Anonymous Nested Class Via Decorator


@autodata(NestedClass)
def test_create_nested_class_using_decorator_returns_not_none(instance):
    assert instance is not None


@autodata(NestedClass)
def test_create_nested_class_using_decorator_returns_instance(instance):
    assert isinstance(instance, NestedClass)


@autodata(NestedClass)
def test_create_nested_class_returns_instance_with_new_values(instance):
    assert instance.id != NestedClass().id
    assert instance.inner.id != SimpleClass().id
    assert instance.inner.text != SimpleClass().text


@autodata()
def test_create_nested_class_argument_returns_instance_with_new_values(
    instance: NestedClass,
):
    assert instance.id != NestedClass().id
    assert instance.inner.id != SimpleClass().id
    assert instance.inner.text != SimpleClass().text


@autodata()
def test_create_anonymous_nested_class_returns_instance_with_new_values(
    instance: NestedClass,
):
    assert instance.id != SimpleClass().id
    assert instance.name != SimpleClass().name
    assert instance.text != SimpleClass().text


@fakedata()
def test_create_fake_nested_class_returns_instance_with_new_values(
    instance: NestedClass,
):
    assert instance.id != NestedClass().id
    assert instance.name != SimpleClass().name
    assert instance.text != SimpleClass().text


# Anonymous Double Nested Class Via Decorator With Fakes


@autodata(DoubleNestedClass, use_fake_data=True)
def test_create_double_nested_class_using_decorator_returns_not_none_with_fake_data(
    instance,
):
    assert instance is not None


@autodata(DoubleNestedClass, use_fake_data=True)
def test_create_double_nested_class_using_decorator_returns_instance_with_fake_data(
    instance,
):
    assert isinstance(instance, DoubleNestedClass)


@autodata(DoubleNestedClass, use_fake_data=True)
def test_create_double_nested_class_returns_instance_with_new_values_with_fake_data(
    instance,
):
    assert instance.id != DoubleNestedClass().id
    assert instance.inner != DoubleNestedClass().inner
    assert instance.inner.id != DoubleNestedClass().id
    assert instance.inner.text != DoubleNestedClass().text


@autodata(use_fake_data=True)
def test_create_double_nested_class_argument_using_decorator_returns_not_none_with_fake_data(
    instance: DoubleNestedClass,
):
    assert instance is not None


@autodata(use_fake_data=True)
def test_create_double_nested_class_argument_using_decorator_returns_instance_with_fake_data(
    instance: DoubleNestedClass,
):
    assert isinstance(instance, DoubleNestedClass)


@fakedata()
def test_create_double_nested_class_argument_returns_instance_with_new_values_with_fake_data(
    instance: DoubleNestedClass,
):
    assert instance.id != DoubleNestedClass().id
    assert instance.inner != DoubleNestedClass().inner
    assert instance.inner.id != SimpleClass().id
    assert instance.inner.text != SimpleClass().text



import unittest
from datetime import date, datetime

from autofaker import autodata, fakedata


class AnonymousPrimitivesViaDecoratorTests(unittest.TestCase):
    @autodata(str)
    def test_create_str_using_decorator(self, text):
        self.assertIsNotNone(text)

    @autodata(int)
    def test_create_int_using_decorator(self, number):
        self.assertIsNotNone(number)

    @autodata(float)
    def test_create_float_using_decorator(self, number):
        self.assertIsNotNone(number)

    @autodata(bool)
    def test_create_boolean_using_decorator(self, boolean):
        self.assertIsNotNone(boolean)

    @autodata(complex)
    def test_create_complex_using_decorator(self, complex_type):
        self.assertIsNotNone(complex_type)

    @autodata(range)
    def test_create_range_using_decorator(self, r):
        self.assertIsNotNone(r)

    @autodata(bytes)
    def test_create_bytes_using_decorator(self, b):
        self.assertIsNotNone(b)

    @autodata(bytearray)
    def test_create_bytearray_using_decorator(self, b):
        self.assertIsNotNone(b)

    @autodata(memoryview)
    def test_create_memoryview_using_decorator(self, b):
        self.assertIsNotNone(b)

    @autodata(datetime)
    def test_create_datetime_using_decorator(self, dt):
        self.assertIsNotNone(dt)

    @autodata(date)
    def test_create_date_using_decorator(self, d):
        self.assertIsNotNone(d)


class AnonymousPrimitivesViaDecoratorWithFakesTests(unittest.TestCase):
    @autodata(str, use_fake_data=True)
    def test_create_str_using_decorator(self, text):
        self.assertIsNotNone(text)

    @autodata(int, use_fake_data=True)
    def test_create_int_using_decorator(self, number):
        self.assertIsNotNone(number)

    @autodata(float, use_fake_data=True)
    def test_create_float_using_decorator(self, number):
        self.assertIsNotNone(number)

    @autodata(bool, use_fake_data=True)
    def test_create_boolean_using_decorator(self, boolean):
        self.assertIsNotNone(boolean)

    @autodata(complex, use_fake_data=True)
    def test_create_complex_using_decorator(self, complex_type):
        self.assertIsNotNone(complex_type)

    @autodata(range, use_fake_data=True)
    def test_create_range_using_decorator(self, r):
        self.assertIsNotNone(r)

    @autodata(bytes, use_fake_data=True)
    def test_create_bytes_using_decorator(self, b):
        self.assertIsNotNone(b)

    @autodata(bytearray, use_fake_data=True)
    def test_create_bytearray_using_decorator(self, b):
        self.assertIsNotNone(b)

    @autodata(memoryview, use_fake_data=True)
    def test_create_memoryview_using_decorator(self, b):
        self.assertIsNotNone(b)

    @autodata(datetime, use_fake_data=True)
    def test_create_datetime_using_decorator(self, dt):
        self.assertIsNotNone(dt)

    @autodata(date, use_fake_data=True)
    def test_create_date_using_decorator(self, d):
        self.assertIsNotNone(d)


class MultipleAnonymousPrimitivesViaDecoratorTests(unittest.TestCase):
    @autodata(str, int, float, complex, range, bytes, bytearray, memoryview)
    def test_create_primitives_using_decorator(
        self, text, number, decimal, complex_type, range_type, buffer, buffer2, memview
    ):
        self.assertIsNotNone(text)
        self.assertNotEqual(number, 0)
        self.assertNotEqual(decimal, float())
        self.assertNotEqual(complex_type, complex())
        self.assertNotEqual(range_type, range(0))
        self.assertNotEqual(buffer, bytes())
        self.assertNotEqual(buffer2, bytearray())
        self.assertNotEqual(memview, memoryview(bytes()))

    @autodata(
        str,
        int,
        float,
        complex,
        range,
        bytes,
        bytearray,
        memoryview,
        use_fake_data=True,
    )
    def test_create_primitives_using_decorator_with_fakes(
        self, text, number, decimal, complex_type, range_type, buffer, buffer2, memview
    ):
        self.assertIsNotNone(text)
        self.assertNotEqual(number, 0)
        self.assertNotEqual(decimal, float())
        self.assertNotEqual(complex_type, complex())
        self.assertNotEqual(range_type, range(0))
        self.assertNotEqual(buffer, bytes())
        self.assertNotEqual(buffer2, bytearray())
        self.assertNotEqual(memview, memoryview(bytes()))


class MultipleArgumentsViaDecoratorTests(unittest.TestCase):
    @fakedata()
    def test_create_anonymous_arguments_using_decorator(
        self,
        text: str,
        number: int,
        decimal: float,
        complex_type: complex,
        range_type: range,
        buffer: bytes,
        buffer2: bytearray,
        memview: memoryview,
    ):
        print(text)
        print(number)
        print(decimal)
        print(complex_type)
        self.assertIsNotNone(text)
        self.assertNotEqual(number, 0)
        self.assertNotEqual(decimal, float())
        self.assertNotEqual(complex_type, complex())
        self.assertNotEqual(range_type, range(0))
        self.assertNotEqual(buffer, bytes())
        self.assertNotEqual(buffer2, bytearray())
        self.assertNotEqual(memview, memoryview(bytes()))

    @fakedata()
    def test_create_fake_arguments_using_decorator(
        self,
        text: str,
        number: int,
        decimal: float,
        complex_type: complex,
        range_type: range,
        buffer: bytes,
        buffer2: bytearray,
        memview: memoryview,
    ):
        print(text)
        print(number)
        print(decimal)
        print(complex_type)
        self.assertIsNotNone(text)
        self.assertNotEqual(number, 0)
        self.assertNotEqual(decimal, float())
        self.assertNotEqual(complex_type, complex())
        self.assertNotEqual(range_type, range(0))
        self.assertNotEqual(buffer, bytes())
        self.assertNotEqual(buffer2, bytearray())
        self.assertNotEqual(memview, memoryview(bytes()))

    @autodata()
    def test_create_arguments_using_decorator(
        self,
        text: str,
        number: int,
        decimal: float,
        complex_type: complex,
        range_type: range,
        buffer: bytes,
        buffer2: bytearray,
        memview: memoryview,
    ):
        print(text)
        print(number)
        print(decimal)
        print(complex_type)
        self.assertIsNotNone(text)
        self.assertNotEqual(number, 0)
        self.assertNotEqual(decimal, float())
        self.assertNotEqual(complex_type, complex())
        self.assertNotEqual(range_type, range(0))
        self.assertNotEqual(buffer, bytes())
        self.assertNotEqual(buffer2, bytearray())
        self.assertNotEqual(memview, memoryview(bytes()))

    @autodata(use_fake_data=True)
    def test_create_arguments_using_decorator_with_fakes(
        self,
        text: str,
        number: int,
        decimal: float,
        complex_type: complex,
        range_type: range,
        buffer: bytes,
        buffer2: bytearray,
        memview: memoryview,
    ):
        print(text)
        print(number)
        print(decimal)
        print(complex_type)
        self.assertIsNotNone(text)
        self.assertNotEqual(number, 0)
        self.assertNotEqual(decimal, float())
        self.assertNotEqual(complex_type, complex())
        self.assertNotEqual(range_type, range(0))
        self.assertNotEqual(buffer, bytes())
        self.assertNotEqual(buffer2, bytearray())
        self.assertNotEqual(memview, memoryview(bytes()))



import unittest
from dataclasses import dataclass

from autofaker import Autodata


@dataclass
class DataClass:
    id: int

    name: str
    first_name: str
    last_name: str
    address: str
    job: str

    country: str
    currency_name: str
    currency_code: str

    email: str
    safe_email: str
    company_email: str

    hostname: str
    ipv4: str
    ipv6: str

    text: str
    word: str


def is_uuid(s: str):
    try:
        from uuid import UUID

        UUID(s)
        return True
    except ValueError:
        return False


class AnonymousDataClassWithFakesTestCase(unittest.TestCase):
    def test_create_many_dataclass_name_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(Autodata.create_many(DataClass, use_fake_data=True)[2].name)
        )

    def test_create_many_dataclass_first_name_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(Autodata.create_many(DataClass, use_fake_data=True)[2].first_name)
        )

    def test_create_many_dataclass_last_name_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(Autodata.create_many(DataClass, use_fake_data=True)[2].last_name)
        )

    def test_create_many_dataclass_address_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(Autodata.create_many(DataClass, use_fake_data=True)[2].address)
        )

    def test_create_many_dataclass_job_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(Autodata.create_many(DataClass, use_fake_data=True)[2].job)
        )

    def test_create_many_dataclass_country_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(Autodata.create_many(DataClass, use_fake_data=True)[2].country)
        )

    def test_create_many_dataclass_currency_name_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(
                Autodata.create_many(DataClass, use_fake_data=True)[2].currency_name
            )
        )

    def test_create_many_dataclass_currency_code_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(
                Autodata.create_many(DataClass, use_fake_data=True)[2].currency_code
            )
        )

    def test_create_many_dataclass_email_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(Autodata.create_many(DataClass, use_fake_data=True)[2].email)
        )

    def test_create_many_dataclass_safe_email_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(Autodata.create_many(DataClass, use_fake_data=True)[2].safe_email)
        )

    def test_create_many_dataclass_company_email_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(
                Autodata.create_many(DataClass, use_fake_data=True)[2].company_email
            )
        )

    def test_create_many_dataclass_hostname_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(Autodata.create_many(DataClass, use_fake_data=True)[2].hostname)
        )

    def test_create_many_dataclass_ipv4_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(Autodata.create_many(DataClass, use_fake_data=True)[2].ipv4)
        )

    def test_create_many_dataclass_ipv6_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(Autodata.create_many(DataClass, use_fake_data=True)[2].ipv6)
        )

    def test_create_many_dataclass_text_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(Autodata.create_many(DataClass, use_fake_data=True)[2].text)
        )

    def test_create_many_dataclass_word_returns_not_uuid(self):
        self.assertFalse(
            is_uuid(Autodata.create_many(DataClass, use_fake_data=True)[2].word)
        )



import unittest
from dataclasses import dataclass

from autofaker import Autodata


class SimpleClassA:
    id = -1
    name = "test"
    text = "test"


class SimpleClassB:
    def __init__(self, id: int, name: str, text: str):
        self.text = text
        self.name = name
        self.id = id


class SimpleClassC:
    def __init__(self, a: SimpleClassA, b: SimpleClassB):
        self.b = b
        self.a = a


class CreateAnonymousPandasDataFrameTests(unittest.TestCase):
    def test_create_anonymous_pandas_dataframe_returns_not_none(self):
        df = Autodata.create_pandas_dataframe(SimpleClassA)
        self.assertIsNotNone(df)

    def test_create_anonymous_pandas_dataframe_returns_not_empty(self):
        df = Autodata.create_pandas_dataframe(SimpleClassA)
        self.assertNotEqual(len(df.index), 0)

    def test_create_anonymous_pandas_dataframe_with_rowcount_returns_not_empty(self):
        df = Autodata.create_pandas_dataframe(SimpleClassA, 10)
        self.assertEqual(len(df.index), 10)

    def test_can_create_anonymous_pandas_dataframes_from_class_with_constructor_arguments(
        self,
    ):
        df = Autodata.create_pandas_dataframe(SimpleClassB)
        self.assertTrue("id" in df.columns)
        self.assertTrue("name" in df.columns)
        self.assertTrue("text" in df.columns)

    def test_can_create_anonymous_pandas_dataframes_from_class_with_constructor_class_arguments(
        self,
    ):
        df = Autodata.create_pandas_dataframe(SimpleClassC)
        self.assertTrue("a" in df.columns)
        self.assertTrue("b" in df.columns)


class CreateFakePandasDataFrameTests(unittest.TestCase):
    def test_create_fake_pandas_dataframe_returns_not_none(self):
        df = Autodata.create_pandas_dataframe(SimpleClassA, use_fake_data=True)
        self.assertIsNotNone(df)

    def test_create_fake_pandas_dataframe_returns_not_empty(self):
        df = Autodata.create_pandas_dataframe(SimpleClassA, use_fake_data=True)
        self.assertNotEqual(len(df.index), 0)

    def test_create_fake_pandas_dataframe_with_rowcount_returns_not_empty(self):
        df = Autodata.create_pandas_dataframe(SimpleClassA, 10, use_fake_data=True)
        self.assertEqual(len(df.index), 10)

    def test_can_create_fake_pandas_dataframes_from_class_with_constructor_arguments(
        self,
    ):
        df = Autodata.create_pandas_dataframe(SimpleClassB, use_fake_data=True)
        self.assertTrue("id" in df.columns)
        self.assertTrue("name" in df.columns)
        self.assertTrue("text" in df.columns)

    def test_can_create_fake_pandas_dataframes_from_class_with_constructor_class_arguments(
        self,
    ):
        df = Autodata.create_pandas_dataframe(SimpleClassC, use_fake_data=True)
        self.assertTrue("a" in df.columns)
        self.assertTrue("b" in df.columns)


@dataclass
class DataClass:
    id: int
    name: str
    text: str


class HybridClassA:
    def __init__(self, a: DataClass, b: SimpleClassA):
        self.b = b
        self.a = a


class HybridClassB:
    def __init__(self, a: DataClass, b: SimpleClassA, c: HybridClassA):
        self.c = c
        self.b = b
        self.a = a


class CreateAnonymousPandasDataFrameFromDataClassTests(unittest.TestCase):
    def test_create_anonymous_pandas_dataframe_returns_not_none(self):
        df = Autodata.create_pandas_dataframe(DataClass)
        self.assertIsNotNone(df)

    def test_create_anonymous_pandas_dataframe_returns_not_empty(self):
        df = Autodata.create_pandas_dataframe(DataClass)
        self.assertNotEqual(len(df.index), 0)

    def test_create_anonymous_pandas_dataframe_with_rowcount_returns_not_empty(self):
        df = Autodata.create_pandas_dataframe(DataClass, 10)
        self.assertEqual(len(df.index), 10)

    def test_can_create_anonymous_pandas_dataframes_from_class_with_constructor_class_arguments(
        self,
    ):
        df = Autodata.create_pandas_dataframe(HybridClassA)
        self.assertTrue("b" in df.columns)
        self.assertTrue("a" in df.columns)

    def test_can_create_anonymous_pandas_dataframes_from_class_with_constructor_hybrid_class_arguments(
        self,
    ):
        df = Autodata.create_pandas_dataframe(HybridClassB)
        self.assertTrue("c" in df.columns)
        self.assertTrue("b" in df.columns)
        self.assertTrue("a" in df.columns)


class CreateFakePandasDataFrameFromDataClassTests(unittest.TestCase):
    def test_create_fake_pandas_dataframe_returns_not_none(self):
        df = Autodata.create_pandas_dataframe(DataClass, use_fake_data=True)
        self.assertIsNotNone(df)

    def test_create_fake_pandas_dataframe_returns_not_empty(self):
        df = Autodata.create_pandas_dataframe(DataClass, use_fake_data=True)
        self.assertNotEqual(len(df.index), 0)

    def test_create_fake_pandas_dataframe_with_rowcount_returns_not_empty(self):
        df = Autodata.create_pandas_dataframe(DataClass, 10, use_fake_data=True)
        self.assertEqual(len(df.index), 10)

    def test_can_create_fake_pandas_dataframes_from_class_with_constructor_class_arguments(
        self,
    ):
        df = Autodata.create_pandas_dataframe(HybridClassA, use_fake_data=True)
        self.assertTrue("b" in df.columns)
        self.assertTrue("a" in df.columns)

    def test_can_create_anonymous_pandas_dataframes_from_class_with_constructor_hybrid_class_arguments(
        self,
    ):
        df = Autodata.create_pandas_dataframe(HybridClassB, use_fake_data=True)
        self.assertTrue("c" in df.columns)
        self.assertTrue("b" in df.columns)
        self.assertTrue("a" in df.columns)



import unittest
from enum import Enum

from autofaker import Autodata


class Weekday(Enum):
    MONDAY = 1
    TUESDAY = 2
    WEDNESDAY = 3
    THURSDAY = 4
    FRIDAY = 5
    SATURDAY = 6
    SUNDAY = 7


class AnonymousWeekdayTestCase(unittest.TestCase):
    def test_create_enum_class_returns_not_none(self):
        self.assertIsNotNone(Autodata.create(Weekday))

    def test_create_enum_class_returns_instance(self):
        self.assertIsInstance(Autodata.create(Weekday), Weekday)


class BasicEnum(Enum):
    """empty"""


class InheritedEnum(BasicEnum):
    VALUE_1 = 1
    VALUE_2 = 2


class AnonymousEnumInheritanceTestCase(unittest.TestCase):
    def test_create_enum_class_returns_not_none(self):
        self.assertIsNotNone(Autodata.create(InheritedEnum))

    def test_create_enum_class_returns_instance(self):
        self.assertIsInstance(Autodata.create(InheritedEnum), BasicEnum)



from autofaker.builtins import (
    BooleanGenerator,
    ByteArrayGenerator,
    BytesGenerator,
    ComplexGenerator,
    FloatGenerator,
    IntegerGenerator,
    MemoryViewGenerator,
    RangeGenerator,
)
from autofaker.fakes import FakeIntegerGenerator, FakeStringGenerator, StringGenerator


class BuiltinTypeDataGeneratorFactory:
    @staticmethod
    def is_supported(type_name) -> bool:
        return type_name in [
            "int",
            "str",
            "float",
            "complex",
            "bool",
            "range",
            "bytes",
            "bytearray",
            "memoryview",
        ]

    @staticmethod
    def create(type_name, field_name: str = None, use_fake_data: bool = False):
        if type_name == "int":
            return (
                FakeIntegerGenerator()
                if field_name is not None and use_fake_data is True
                else IntegerGenerator()
            )
        if type_name == "str":
            return (
                FakeStringGenerator(field_name)
                if field_name is not None and use_fake_data is True
                else StringGenerator()
            )
        if type_name == "float":
            return FloatGenerator()
        if type_name == "complex":
            return ComplexGenerator()
        if type_name == "bool":
            return BooleanGenerator()
        if type_name == "range":
            return RangeGenerator()
        if type_name == "bytes":
            return BytesGenerator()
        if type_name == "bytearray":
            return ByteArrayGenerator()
        if type_name == "memoryview":
            return MemoryViewGenerator()



import unittest
from abc import abstractmethod
from dataclasses import dataclass
from datetime import date, datetime
from typing import List

from autofaker import Autodata


class CreateTestCase(unittest.TestCase):
    @abstractmethod
    def getType(self):
        pass

    def create(self):
        return Autodata.create(self.getType())

    def test_create_returns_not_none(self):
        if self.getType() is None:
            return
        self.assertIsNotNone(self.create())

    def test_create_returns_not_type(self):
        if self.getType() is None:
            return
        self.assertNotIsInstance(self.create(), type)

    def test_create_returns_not_default(self):
        if self.getType() is None or self.getType() is bool:
            return
        collection = self.create()
        self.assertNotEqual(collection, [])

    def test_create_returns_not_empty(self):
        if self.getType() is None or self.getType() is bool:
            return
        collection = self.create()
        self.assertNotEqual(len(collection), 0)


class AnonymousIntegerListTestCase(CreateTestCase):
    def getType(self):
        return List[int]


class AnonymousStringListTestCase(CreateTestCase):
    def getType(self):
        return List[str]


class AnonymousFloatListTestCase(CreateTestCase):
    def getType(self):
        return List[float]


class AnonymousBooleanListTestCase(CreateTestCase):
    def getType(self):
        return List[bool]


class AnonymousComplexListTestCase(CreateTestCase):
    def getType(self):
        return List[complex]


class AnonymousDatetimeListTestCase(CreateTestCase):
    def getType(self):
        return List[datetime]


class AnonymousDateListTestCase(CreateTestCase):
    def getType(self):
        return List[date]


class AnonymousNestedListTestCase(CreateTestCase):
    def getType(self):
        return List[List[int]]


class AnonymousDoubleNestedListTestCase(CreateTestCase):
    def getType(self):
        return List[List[List[int]]]


@dataclass
class DataClass:
    name: str
    age: int


class AnonymousDataClassListTestCase(CreateTestCase):
    def getType(self):
        return List[DataClass]

    def test_name_not_none(self):
        for data in self.create():
            self.assertIsNotNone(data.name)

    def test_age_not_default(self):
        for data in self.create():
            self.assertNotEqual(data.age, int())


@dataclass
class NestedDataClass:
    a: DataClass
    b: DataClass
    c: DataClass


class AnonymousNestedDataClassListTestCase(CreateTestCase):
    def getType(self):
        return List[NestedDataClass]

    def test_name_not_none(self):
        for data in self.create():
            self.assertIsNotNone(data.a.name)
            self.assertIsNotNone(data.b.name)
            self.assertIsNotNone(data.c.name)

    def test_age_not_default(self):
        for data in self.create():
            self.assertNotEqual(data.a.age, int())
            self.assertIsNotNone(data.b.age, int())
            self.assertIsNotNone(data.c.age, int())


class ConstructorWithNestedDataClassList:
    def __init__(self, inner: List[NestedDataClass]):
        self.inner = inner


class AnonymousDoubleNestedDataClassListTestCase(CreateTestCase):
    def getType(self):
        return List[ConstructorWithNestedDataClassList]

    def test_inner_not_none(self):
        for data in self.create():
            self.assertIsNotNone(data.inner)

    def test_inner_not_empty(self):
        for data in self.create():
            self.assertNotEqual(len(data.inner), 0)



import unittest
from enum import Enum

from autofaker import autodata


class Weekday(Enum):
    MONDAY = 1
    TUESDAY = 2
    WEDNESDAY = 3
    THURSDAY = 4
    FRIDAY = 5
    SATURDAY = 6
    SUNDAY = 7


class AnonymousWeekdayTestCase(unittest.TestCase):
    @autodata(Weekday)
    def test_create_enum_class_returns_not_none(self, sut):
        self.assertIsNotNone(sut)

    @autodata(Weekday)
    def test_create_enum_class_returns_instance(self, sut):
        self.assertIsInstance(sut, Weekday)



"""
Provides modules for creation anonymous variables to help minimize the setup/arrange phase when writing unit tests
"""

__version__ = "0.1.0"

from autofaker.autodata import *
from autofaker.decorators import *



import unittest
from dataclasses import dataclass

import pandas

from autofaker import autodata, autopandas, fakepandas


class SimpleClassA:
    id = -1
    name: "test"
    text = "test"


class SimpleClassB:
    def __init__(self, id: int, name: str, text: str):
        self.text = text
        self.name = name
        self.id = id


class SimpleClassC:
    def __init__(self, a: SimpleClassA, b: SimpleClassB):
        self.b = b
        self.a = a


class AnonymousPandasDataFrameViaDecoratorTests(unittest.TestCase):
    @autopandas(SimpleClassA)
    def test_create_anonymous_pandas_dataframe_returns_not_none(
        self, df: pandas.DataFrame
    ):
        self.assertIsNotNone(df)

    @autopandas(SimpleClassA, 10)
    def test_create_anonymous_pandas_dataframe_with_rowcount_returns_not_empty(
        self, df: pandas.DataFrame
    ):
        self.assertNotEqual(len(df.index), 0)

    @fakepandas(SimpleClassA)
    def test_create_fake_pandas_dataframe_returns_not_none(self, df: pandas.DataFrame):
        self.assertIsNotNone(df)

    @fakepandas(SimpleClassA, 10)
    def test_create_fake_pandas_dataframe_with_rowcount_returns_not_empty(
        self, df: pandas.DataFrame
    ):
        self.assertNotEqual(len(df.index), 0)

    @autopandas(SimpleClassA)
    def test_can_create_anonymous_pandas_dataframes(self, cls):
        self.assertIsNotNone(cls)

    @autopandas(SimpleClassB)
    def test_can_create_anonymous_pandas_dataframes_from_class_with_constructor_arguments(
        self, cls
    ):
        self.assertIsNotNone(cls)

    @autopandas(SimpleClassC)
    def test_can_create_anonymous_pandas_dataframes_from_class_with_constructor_class_arguments(
        self, cls
    ):
        self.assertIsNotNone(cls)


@dataclass
class DataClass:
    id: int
    name: str
    text: str


class AnonymousPandasDataFrameViaDecoratorFromDataClassTests(unittest.TestCase):
    @autopandas(DataClass)
    def test_create_anonymous_pandas_dataframe_returns_not_none(
        self, df: pandas.DataFrame
    ):
        self.assertIsNotNone(df)

    @autopandas(DataClass, 10)
    def test_create_anonymous_pandas_dataframe_with_rowcount_returns_not_empty(
        self, df: pandas.DataFrame
    ):
        self.assertNotEqual(len(df.index), 0)

    @fakepandas(DataClass, 10)
    def test_create_fake_pandas_dataframe_with_rowcount_returns_not_empty(
        self, df: pandas.DataFrame
    ):
        self.assertNotEqual(len(df.index), 0)

    @fakepandas(DataClass)
    def test_create_fake_pandas_dataframe_returns_not_none(self, df: pandas.DataFrame):
        self.assertIsNotNone(df)


class AutodataDecoratorIgnoresPandas(unittest.TestCase):
    @autodata()
    def test_autodata_decorator_ignores_pandas_dataframe(self, df: pandas.DataFrame):
        self.assertIsNone(df)

    @autodata()
    def test_autodata_decorator_ignores_only_pandas_dataframe(
        self, text: str, df: pandas.DataFrame
    ):
        self.assertIsNotNone(text)
        self.assertIsNone(df)



import random
from enum import Enum

from autofaker.base import TypeDataGeneratorBase


def is_enum(t) -> bool:
    return isinstance(t, Enum.__class__)


class EnumGenerator(TypeDataGeneratorBase):
    def __init__(self, enum):
        self.enum = enum

    def generate(self):
        names = [member.name for member in self.enum]
        value = names[random.randint(0, len(self.enum) - 1)]
        return self.enum[value]



"""
Provides classes for anonymous object creation to help minimize the setup/arrange phase when writing unit tests
"""

import typing
from typing import List

from autofaker.dataframe import PandasDataFrameGenerator
from autofaker.generator import TypeDataGenerator


class Autodata:
    """
    Provides anonymous object creation functions to help minimize the setup/arrange phase when writing unit tests
    """

    @staticmethod
    def create(t, use_fake_data: bool = False):
        """
        Creates an anonymous variable of the requested type

        :param type t: type - The type to generate data for
        :param use_fake_data: bool - Set this to True to use Faker to generate data, otherwise False to generate anonymous data
        """
        return TypeDataGenerator.create(t, use_fake_data=use_fake_data).generate()

    @staticmethod
    def create_many(t, size: int = 3, use_fake_data: bool = False) -> List[typing.Any]:
        """
        Creates a list of anonymous variables of the requested type using the specified length (default 3)

        :param type t: type - The type to generate data for
        :type size: int - The number of items to generate (default 3)
        :param use_fake_data: bool - Set this to True to use Faker to generate data, otherwise False to generate anonymous data
        """
        items = []
        for _ in range(size):
            items.append(Autodata.create(t, use_fake_data=use_fake_data))
        return items

    @staticmethod
    def create_pandas_dataframe(t, rows: int = 3, use_fake_data: bool = False):
        """
        Create a Pandas DataFrame containing anonymous data with the specified number of rows (default 3)

        :param type t: object - The class that represents the DataFrame. This can be a plain old class or a @dataclass
        :param type rows: int - The number of rows to generate for the DataFrame (default 3)
        :param use_fake_data: bool - Set this to True to use Faker to generate data, otherwise False to generate anonymous data
        """
        return PandasDataFrameGenerator(t, rows, use_fake_data=use_fake_data).generate()



import setuptools

with open("docs/pypi.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="autofaker",
    version='1.0.18',
    url="https://github.com/christianhelle/autofaker",
    license="MIT License",
    license_files=["LICENSE"],
    author="Christian Helle",
    author_email="christian.helle@outlook.com",
    description="Python library designed to minimize the setup/arrange phase of your unit tests",
    long_description=long_description,
    long_description_content_type="text/markdown",
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.7",
    install_requires=["pandas", "faker", "typing_inspect"],
)



import unittest
from dataclasses import dataclass
from typing import List

from autofaker import Autodata


@dataclass
class Person:
    age: int
    name: str


class Staff:
    def __init__(self, people: List[Person] = None):
        self.people = people if people is not None else []

    def add_person(self, person: Person):
        self.people.append(person)

    def add_people(self, people: List[Person]):
        for person in people:
            self.add_person(person)

    def get_head_count(self):
        return len(self.people)


class Organization:
    def __init__(self, staff: Staff):
        self.staff = staff

    def get_staff(self):
        return self.staff


class ConstructorWithPrimitiveArguments:
    def __init__(self, name: str, text: str, age: int):
        self.age = age
        self.text = text
        self.name = name


class ConstructorWithPrimitiveArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithPrimitiveArguments))

    def test_constructed_sut_age_not_none(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithPrimitiveArguments).age)

    def test_constructed_sut_name_not_none(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithPrimitiveArguments).name)

    def test_constructed_sut_text_not_none(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithPrimitiveArguments).text)


class ConstructorWithClassArguments:
    def __init__(self, cls: ConstructorWithPrimitiveArguments):
        self.inner = cls


class ConstructorWithClassArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithClassArguments))

    def test_constructed_sut_inner_not_none(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithClassArguments).inner)

    def test_constructed_sut_inner_has_correct_type(self):
        self.assertIsInstance(
            Autodata.create(ConstructorWithClassArguments).inner,
            ConstructorWithPrimitiveArguments,
        )


class ConstructorWithNestedClassArguments:
    def __init__(self, cls: ConstructorWithClassArguments):
        self.inner = cls


class ConstructorWithNestedClassArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithNestedClassArguments))

    def test_constructed_sut_inner_not_none(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithNestedClassArguments).inner)

    def test_constructed_sut_inner_has_correct_type(self):
        self.assertIsInstance(
            Autodata.create(ConstructorWithNestedClassArguments).inner,
            ConstructorWithClassArguments,
        )

    def test_constructed_sut_inner_inner_has_correct_type(self):
        self.assertIsInstance(
            Autodata.create(ConstructorWithNestedClassArguments).inner.inner,
            ConstructorWithPrimitiveArguments,
        )


class ConstructorWithDoubleNestedClassArguments:
    def __init__(self, cls: ConstructorWithNestedClassArguments):
        self.inner = cls


class ConstructorWithDoubleNestedClassArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithDoubleNestedClassArguments))

    def test_constructed_sut_inner_not_none(self):
        self.assertIsNotNone(
            Autodata.create(ConstructorWithDoubleNestedClassArguments).inner
        )

    def test_constructed_sut_inner_has_correct_type(self):
        self.assertIsInstance(
            Autodata.create(ConstructorWithDoubleNestedClassArguments).inner,
            ConstructorWithNestedClassArguments,
        )

    def test_constructed_sut_inner_inner_has_correct_type(self):
        self.assertIsInstance(
            Autodata.create(ConstructorWithDoubleNestedClassArguments).inner.inner,
            ConstructorWithClassArguments,
        )

    def test_constructed_sut_inner_inner_inner_has_correct_type(self):
        self.assertIsInstance(
            Autodata.create(
                ConstructorWithDoubleNestedClassArguments
            ).inner.inner.inner,
            ConstructorWithPrimitiveArguments,
        )


@dataclass
class DataClass:
    id: int
    name: str
    age: int
    email: str


class ConstructorWithDataClassArguments:
    def __init__(self, data: DataClass):
        self.data = data


class ConstructorWithDataClassArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithDataClassArguments))

    def test_constructed_sut_id_not_none(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithDataClassArguments).data.id)

    def test_constructed_sut_age_not_none(self):
        self.assertIsNotNone(
            Autodata.create(ConstructorWithDataClassArguments).data.age
        )

    def test_constructed_sut_name_not_none(self):
        self.assertIsNotNone(
            Autodata.create(ConstructorWithDataClassArguments).data.name
        )

    def test_constructed_sut_email_not_none(self):
        self.assertIsNotNone(
            Autodata.create(ConstructorWithDataClassArguments).data.email
        )


class ConstructorWithListArguments:
    def __init__(self, texts: List[str], numbers: List[int]):
        self.numbers = numbers
        self.texts = texts


class ConstructorWithListArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithListArguments))


class ConstructorWithClassListArguments:
    def __init__(self, items: List[ConstructorWithPrimitiveArguments]):
        self.items = items


class ConstructorWithClassListArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithClassListArguments))


class ConstructorWithDataClassListArguments:
    def __init__(self, items: List[DataClass]):
        self.items = items


class ConstructorWithDataClassListArgumentsTests(unittest.TestCase):
    def test_can_construct_sut(self):
        self.assertIsNotNone(Autodata.create(ConstructorWithDataClassListArguments))



import datetime
import random

from autofaker.base import TypeDataGeneratorBase


def is_date_type(type_name) -> bool:
    return type_name in ["datetime", "date"]


class DatetimeGenerator(TypeDataGeneratorBase):
    def generate(self):
        year = datetime.date.today().year
        return datetime.datetime(
            random.randint(year - 10, year + 10),
            random.randint(1, 12),
            random.randint(1, 28),
            random.randint(0, 23),
            random.randint(0, 59),
            random.randint(0, 59),
            random.randint(0, 999),
        )


class DateGenerator(TypeDataGeneratorBase):
    def generate(self):
        year = datetime.date.today().year
        return datetime.datetime(
            random.randint(year - 10, year + 10),
            random.randint(1, 12),
            random.randint(1, 28),
        )



from enum import Enum

from autofaker import autodata


class Weekday(Enum):
    MONDAY = 1
    TUESDAY = 2
    WEDNESDAY = 3
    THURSDAY = 4
    FRIDAY = 5
    SATURDAY = 6
    SUNDAY = 7


@autodata(Weekday)
def test_create_enum_class_returns_not_none(sut):
    assert sut is not None


@autodata(Weekday)
def test_create_enum_class_returns_instance(sut):
    assert isinstance(sut, Weekday)


class BasicEnum(Enum):
    """empty"""


class InheritedEnum(BasicEnum):
    VALUE_1 = 1
    VALUE_2 = 2


@autodata(InheritedEnum)
def test_create_enum_class_returns_not_none(sut):
    assert sut is not None


@autodata(InheritedEnum)
def test_create_enum_class_returns_instance(sut):
    assert isinstance(sut, BasicEnum)



import dataclasses
from dataclasses import dataclass

from autofaker import autodata, fakedata


@dataclass
class DataClass:
    id: int
    name: str
    text: str


# Anonymous DataClass Via Decorator Test Cases


@autodata(DataClass)
def test_create_data_class_using_decorator_returns_not_none(instance):
    assert instance is not None


@autodata(DataClass)
def test_create_data_class_using_decorator_returns_dataclass(instance):
    assert dataclasses.is_dataclass(instance)


@autodata(DataClass)
def test_create_data_class_returns_instance_with_new_values(instance):
    assert instance.id != int()
    assert instance.name != str()
    assert instance.text != str()


@autodata()
def test_create_data_class_argument_returns_instance_with_new_values(
    instance: DataClass,
):
    assert instance.id != int()
    assert instance.name != str()
    assert instance.text != str()


@autodata()
def test_create_anonymous_data_class_returns_instance_with_new_values(
    instance: DataClass,
):
    assert instance.id != int()
    assert instance.name != str()
    assert instance.text != str()


@fakedata()
def test_create_fake_data_class_returns_instance_with_new_values(instance: DataClass):
    assert instance.id != int()
    assert instance.name != str()
    assert instance.text != str()


# Anonymous DataClass Via Decorator With Fakes Test Cases


@autodata(DataClass, use_fake_data=True)
def test_create_data_class_using_decorator_returns_not_none_with_fake_data(instance):
    assert instance is not None


@autodata(DataClass, use_fake_data=True)
def test_create_data_class_using_decorator_returns_dataclass_with_fake_data(instance):
    assert dataclasses.is_dataclass(instance)


@autodata(DataClass, use_fake_data=True)
def test_create_data_class_returns_instance_with_new_values_with_fake_data(instance):
    assert instance.id != int()
    assert instance.text != str()


@autodata(use_fake_data=True)
def test_create_data_class_argument_returns_instance_with_new_values_with_fake_data(
    instance: DataClass,
):
    assert instance.id != int()
    assert instance.name != str()
    assert instance.text != str()


@fakedata()
def test_create_fake_data_class_returns_instance_with_new_values_with_fake_data(
    instance: DataClass,
):
    assert instance.id != int()
    assert instance.name != str()
    assert instance.text != str()



import dataclasses
import inspect

import typing_inspect

from autofaker.attributes import Attributes
from autofaker.dates import DateGenerator, DatetimeGenerator, is_date_type
from autofaker.enums import EnumGenerator, is_enum
from autofaker.factory import BuiltinTypeDataGeneratorFactory
from autofaker.fakes import TypeDataGeneratorBase


class TypeDataGenerator:
    @staticmethod
    def create(
        t, field_name: str = None, use_fake_data: bool = False
    ) -> TypeDataGeneratorBase:
        type_name = TypeDataGenerator._get_type_name(t).lower()
        if BuiltinTypeDataGeneratorFactory.is_supported(type_name):
            return BuiltinTypeDataGeneratorFactory.create(
                type_name, field_name, use_fake_data
            )
        if is_date_type(type_name):
            return TypeDataGenerator.create_datetime(
                type_name, field_name, use_fake_data
            )
        if type_name == "list":
            return ListGenerator(t)
        if is_enum(t):
            return EnumGenerator(t)
        return (
            DataClassGenerator(t, use_fake_data=use_fake_data)
            if dataclasses.is_dataclass(t)
            else ClassGenerator(t, use_fake_data=use_fake_data)
        )

    @staticmethod
    def create_datetime(type_name, field_name: str = None, use_fake_data: bool = False):
        if type_name == "datetime":
            return DatetimeGenerator()
        if type_name == "date":
            return DateGenerator()

    @staticmethod
    def _get_type_name(t) -> str:
        PRIMITIVE_TYPES = {
            "int", "float", "str", "complex", "range", "bytes", "bytearray"
        }

        try:
            return t.__name__
        except Exception:
            attributes = dir(t)
            if "_name" in attributes:
                return t._name
            # If __future__.annotations was imported by the user, then the type
            # will be a str. Thus, asserting the type with type() will fail,
            # because it will always be a string. This is because, annotations
            # transforms any type into a string object. Therefore, if the type
            # is a string, assess if it is the name of a known primitive type.
            elif type(t) == str and t in PRIMITIVE_TYPES:
                return t
            return type(t).__name__


class DataClassGenerator(TypeDataGeneratorBase):
    def __init__(self, cls, use_fake_data: bool = False):
        self.use_fake_data = use_fake_data
        self.cls = cls

    def generate(self):
        args = []
        fields = dataclasses.fields(self.cls)
        for field in fields:
            generator = TypeDataGenerator.create(
                field.type, field.name, use_fake_data=self.use_fake_data
            )
            args.append((field.name, field.type, generator.generate()))
        name = self.cls.__module__ + "." + self.cls.__qualname__
        instance = dataclasses.make_dataclass(name, args)
        return instance


class ClassGenerator(TypeDataGeneratorBase):
    def __init__(self, cls, use_fake_data: bool = False):
        self.use_fake_data = use_fake_data
        self._try_create_instance(cls)

    def generate(self):
        if not self._is_supported():
            return None

        attributes = Attributes(self.instance)
        members = attributes.get_members()
        for member in members:
            attr = attributes.get_attribute(member)
            if type(attr).__name__ == "list":
                for i in range(len(attr)):
                    attr[i] = self._try_generate(attr[i])
            else:
                attributes.set_value(member, self._try_generate(attr))
        return self.instance

    def _is_supported(self):
        import pandas

        unsupported_types = [pandas.core.frame.DataFrame]
        for t in unsupported_types:
            if isinstance(self.instance, t):
                return False
        return True

    def _try_generate(self, attr):
        try:
            generator = TypeDataGenerator.create(
                type(attr), use_fake_data=self.use_fake_data
            )
            return generator.generate()
        except TypeError as e:
            print(e)

    def _try_create_instance(self, cls):
        try:
            self.instance = cls()
        except TypeError:
            self._create_with_init_args(cls)

    def _create_with_init_args(self, cls):
        init_args = inspect.getfullargspec(cls.__init__)
        values = []
        for t in init_args.annotations.values():
            origin = typing_inspect.get_origin(t)
            if origin == list:
                list_arg = typing_inspect.get_args(t)
                items = []
                for _ in range(3):
                    generator = TypeDataGenerator.create(
                        list_arg[0], use_fake_data=self.use_fake_data
                    )
                    items.append(generator.generate())
                values.append(items)
            else:
                generator = TypeDataGenerator.create(
                    t, use_fake_data=self.use_fake_data
                )
                value = generator.generate()
                values.append(value)
        self.instance = cls(*tuple(values))


class ListGenerator(TypeDataGeneratorBase):
    def __init__(self, t, use_fake_data: bool = False):
        self.use_fake_data = use_fake_data
        self.list_arg = typing_inspect.get_args(t)

    def generate(self):
        items = []
        for _ in range(3):
            generator = TypeDataGenerator.create(
                self.list_arg[0], use_fake_data=self.use_fake_data
            )
            items.append(generator.generate())
        return items



import dataclasses
import unittest
from dataclasses import dataclass

from autofaker import autodata, fakedata


@dataclass
class SimpleClass:
    id: int
    name: str
    text: str


@dataclass
class NestedClass:
    id: int
    name: str
    text: str
    inner: SimpleClass


@dataclass
class DoubleNestedClass:
    id: int
    name: str
    text: str
    inner: NestedClass


class AnonymousNestedClassViaDecoratorTestCase(unittest.TestCase):
    @autodata(NestedClass)
    def test_create_nested_class_using_decorator_returns_not_none(self, instance):
        self.assertIsNotNone(instance)

    @autodata(NestedClass)
    def test_create_nested_class_using_decorator_returns_dataclass(self, instance):
        self.assertTrue(dataclasses.is_dataclass(instance))

    @autodata(NestedClass)
    def test_create_nested_class_returns_instance_with_new_values(self, instance):
        self.assertNotEqual(instance.id, 0)
        self.assertNotEqual(instance.inner.id, 0)
        self.assertNotEqual(instance.inner.text, str())

    @autodata()
    def test_create_nested_class_argument_returns_instance_with_new_values(
        self, instance: NestedClass
    ):
        self.assertNotEqual(instance.id, 0)
        self.assertNotEqual(instance.name, str())
        self.assertNotEqual(instance.text, str())

    @autodata()
    def test_create_anonymous_nested_class_returns_instance_with_new_values(
        self, instance: NestedClass
    ):
        self.assertNotEqual(instance.id, 0)
        self.assertNotEqual(instance.name, str())
        self.assertNotEqual(instance.text, str())

    @fakedata()
    def test_create_fake_nested_class_returns_instance_with_new_values(
        self, instance: NestedClass
    ):
        self.assertNotEqual(instance.id, 0)
        self.assertNotEqual(instance.name, str())
        self.assertNotEqual(instance.text, str())


class AnonymousDoubleNestedClassViaDecoratorWithFakesTestCase(unittest.TestCase):
    @autodata(DoubleNestedClass, use_fake_data=True)
    def test_create_double_nested_class_using_decorator_returns_not_none(
        self, instance
    ):
        self.assertIsNotNone(instance)

    @autodata(DoubleNestedClass, use_fake_data=True)
    def test_create_nested_class_using_decorator_returns_dataclass(self, instance):
        self.assertTrue(dataclasses.is_dataclass(instance))

    @autodata(DoubleNestedClass, use_fake_data=True)
    def test_create_double_nested_class_returns_instance_with_new_values(
        self, instance
    ):
        self.assertNotEqual(instance.id, 0)
        self.assertNotEqual(instance.inner.inner.id, 0)
        self.assertNotEqual(instance.inner.inner.text, str())

    @autodata(use_fake_data=True)
    def test_create_double_nested_class_argument_using_decorator_returns_not_none(
        self, instance: DoubleNestedClass
    ):
        self.assertIsNotNone(instance)

    @fakedata()
    def test_create_double_nested_class_argument_returns_instance_with_new_values(
        self, instance: DoubleNestedClass
    ):
        self.assertNotEqual(instance.id, 0)
        self.assertNotEqual(instance.inner.inner.id, 0)
        self.assertNotEqual(instance.inner.inner.text, str())
